@@ -19,8 +19,9 @@
 #include "hpdf_utils.h"
 #include "hpdf_image.h"
 
-#ifndef LIBHPDF_HAVE_NOPNGLIB
+#ifdef LIBHPDF_HAVE_LIBPNG
 #include <png.h>
+#include <string.h>
 
 static void
 PngErrorFunc  (png_structp       png_ptr,
@@ -109,14 +110,15 @@ ReadPngData_Interlaced  (HPDF_Dict    image,
                          png_infop    info_ptr)
 {
     png_uint_32 len = png_get_rowbytes(png_ptr, info_ptr);
+    png_uint_32 height = png_get_image_height(png_ptr, info_ptr);
     png_bytep* row_pointers = HPDF_GetMem (image->mmgr,
-                info_ptr->height * sizeof (png_bytep));
+                height * sizeof (png_bytep));
 
     if (row_pointers) {
         HPDF_UINT i;
 
-        HPDF_MemSet (row_pointers, 0, info_ptr->height * sizeof (png_bytep));
-        for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+        HPDF_MemSet (row_pointers, 0, height * sizeof (png_bytep));
+        for (i = 0; i < (HPDF_UINT)height; i++) {
             row_pointers[i] = HPDF_GetMem (image->mmgr, len);
 
             if (image->error->error_no != HPDF_OK)
@@ -126,7 +128,7 @@ ReadPngData_Interlaced  (HPDF_Dict    image,
         if (image->error->error_no == HPDF_OK) {
             png_read_image(png_ptr, row_pointers);
             if (image->error->error_no == HPDF_OK) {       /* add this line */
-                for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+                for (i = 0; i < (HPDF_UINT)height; i++) {
                     if (HPDF_Stream_Write (image->stream, row_pointers[i], len) !=
                             HPDF_OK)
                         break;
@@ -135,7 +137,7 @@ ReadPngData_Interlaced  (HPDF_Dict    image,
         }
 
         /* clean up */
-        for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+        for (i = 0; i < (HPDF_UINT)height; i++) {
             HPDF_FreeMem (image->mmgr, row_pointers[i]);
         }
 
@@ -151,12 +153,13 @@ ReadPngData  (HPDF_Dict    image,
               png_infop    info_ptr)
 {
     png_uint_32 len = png_get_rowbytes(png_ptr, info_ptr);
+    png_uint_32 height = png_get_image_height(png_ptr, info_ptr);
     png_bytep buf_ptr = HPDF_GetMem (image->mmgr, len);
 
     if (buf_ptr) {
         HPDF_UINT i;
 
-        for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+        for (i = 0; i < (HPDF_UINT)height; i++) {
             png_read_rows(png_ptr, (png_byte**)&buf_ptr, NULL, 1);
             if (image->error->error_no != HPDF_OK)
                 break;
@@ -182,17 +185,19 @@ ReadTransparentPaletteData  (HPDF_Dict    image,
 	HPDF_STATUS ret = HPDF_OK;
 	HPDF_UINT i, j;
 	png_bytep *row_ptr;
+	png_uint_32 height = png_get_image_height(png_ptr, info_ptr);
+	png_uint_32 width = png_get_image_width(png_ptr, info_ptr);
 
-	row_ptr = HPDF_GetMem (image->mmgr, info_ptr->height * sizeof(png_bytep));
+	row_ptr = HPDF_GetMem (image->mmgr, height * sizeof(png_bytep));
 	if (!row_ptr) {
 		return HPDF_FAILD_TO_ALLOC_MEM;
 	} else {
 		png_uint_32 len = png_get_rowbytes(png_ptr, info_ptr);
 
-		for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+		for (i = 0; i < (HPDF_UINT)height; i++) {
 			row_ptr[i] = HPDF_GetMem(image->mmgr, len);
 			if (!row_ptr[i]) {
-				for (; i >= 0; i--) {
+				for (; i > 0; i--) {
 					HPDF_FreeMem (image->mmgr, row_ptr[i]);
 				}
 				HPDF_FreeMem (image->mmgr, row_ptr);
@@ -207,19 +212,19 @@ ReadTransparentPaletteData  (HPDF_Dict    image,
 		goto Error;
 	}
 
-	for (j = 0; j < info_ptr->height; j++) {
-		for (i = 0; i < info_ptr->width; i++) {
-			smask_data[info_ptr->width * j + i] = (row_ptr[j][i] < num_trans) ? trans[row_ptr[j][i]] : 0xFF;
+	for (j = 0; j < height; j++) {
+		for (i = 0; i < width; i++) {
+			smask_data[width * j + i] = (row_ptr[j][i] < num_trans) ? trans[row_ptr[j][i]] : 0xFF;
 		}
 
-		if (HPDF_Stream_Write (image->stream, row_ptr[j], info_ptr->width) != HPDF_OK) {
+		if (HPDF_Stream_Write (image->stream, row_ptr[j], width) != HPDF_OK) {
 			ret = HPDF_FILE_IO_ERROR;
 			goto Error;
 		}
 	}
 
 Error:
-	for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+	for (i = 0; i < (HPDF_UINT)height; i++) {
 		HPDF_FreeMem (image->mmgr, row_ptr[i]);
 	}
 
@@ -238,6 +243,8 @@ ReadTransparentPngData  (HPDF_Dict    image,
 	HPDF_UINT i, j;
 	png_bytep *row_ptr, row;
 	png_byte color_type;
+	png_uint_32 height = png_get_image_height(png_ptr, info_ptr);
+	png_uint_32 width = png_get_image_width(png_ptr, info_ptr);
 
 	color_type = png_get_color_type(png_ptr, info_ptr);
 
@@ -245,16 +252,16 @@ ReadTransparentPngData  (HPDF_Dict    image,
 		return HPDF_INVALID_PNG_IMAGE;
 	}
 
-	row_ptr = HPDF_GetMem (image->mmgr, info_ptr->height * sizeof(png_bytep));
+	row_ptr = HPDF_GetMem (image->mmgr, height * sizeof(png_bytep));
 	if (!row_ptr) {
 		return HPDF_FAILD_TO_ALLOC_MEM;
 	} else {
 		png_uint_32 len = png_get_rowbytes(png_ptr, info_ptr);
 
-		for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+		for (i = 0; i < (HPDF_UINT)height; i++) {
 			row_ptr[i] = HPDF_GetMem(image->mmgr, len);
 			if (!row_ptr[i]) {
-				for (; i >= 0; i--) {
+				for (; i > 0; i--) {
 					HPDF_FreeMem (image->mmgr, row_ptr[i]);
 				}
 				HPDF_FreeMem (image->mmgr, row_ptr);
@@ -271,12 +278,12 @@ ReadTransparentPngData  (HPDF_Dict    image,
 
 	switch (color_type) {
 		case PNG_COLOR_TYPE_RGB_ALPHA:
-			row_len = 3 * info_ptr->width * sizeof(png_byte);
-			for (j = 0; j < info_ptr->height; j++) {
-				for (i = 0; i < info_ptr->width; i++) {
-					row = row_ptr[j];
+			row_len = 3 * width * sizeof(png_byte);
+			for (j = 0; j < height; j++) {
+				row = row_ptr[j];
+				for (i = 0; i < width; i++) {
 					memmove(row + (3 * i), row + (4*i), 3);
-					smask_data[info_ptr->width * j + i] = row[4 * i + 3];
+					smask_data[width * j + i] = row[4 * i + 3];
 				}
 
 				if (HPDF_Stream_Write (image->stream, row, row_len) != HPDF_OK) {
@@ -286,12 +293,12 @@ ReadTransparentPngData  (HPDF_Dict    image,
 			}
 			break;
 		case PNG_COLOR_TYPE_GRAY_ALPHA:
-			row_len = info_ptr->width * sizeof(png_byte);
-			for (j = 0; j < info_ptr->height; j++) {
-				for (i = 0; i < info_ptr->width; i++) {
-					row = row_ptr[j];
+			row_len = width * sizeof(png_byte);
+			for (j = 0; j < height; j++) {
+				row = row_ptr[j];
+				for (i = 0; i < width; i++) {
 					row[i] = row[2 * i];
-					smask_data[info_ptr->width * j + i] = row[2 * i + 1];
+					smask_data[width * j + i] = row[2 * i + 1];
 				}
 
 				if (HPDF_Stream_Write (image->stream, row, row_len) != HPDF_OK) {
@@ -306,7 +313,7 @@ ReadTransparentPngData  (HPDF_Dict    image,
 	}
 
 Error:
-	for (i = 0; i < (HPDF_UINT)info_ptr->height; i++) {
+	for (i = 0; i < (HPDF_UINT)height; i++) {
 		HPDF_FreeMem (image->mmgr, row_ptr[i]);
 	}
 
@@ -415,7 +422,8 @@ LoadPngData  (HPDF_Dict     image,
 
 {
 	HPDF_STATUS ret = HPDF_OK;
-
+	png_uint_32 width, height;
+	int bit_depth, color_type;
 	png_structp png_ptr = NULL;
 	png_infop info_ptr = NULL;
 
@@ -447,9 +455,12 @@ LoadPngData  (HPDF_Dict     image,
 		goto Exit;
 	}
 
+	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, NULL, NULL, NULL);
+
 	/* 16bit images are not supported. */
-	if (info_ptr->bit_depth == 16) {
+	if (bit_depth == 16) {
 		png_set_strip_16(png_ptr);
+		bit_depth = 8;
 	}
 
 	png_read_update_info(png_ptr, info_ptr);
@@ -458,13 +469,13 @@ LoadPngData  (HPDF_Dict     image,
 	}
 
 	/* check palette-based images for transparent areas and load them immediately if found */
-	if (xref && PNG_COLOR_TYPE_PALETTE & info_ptr->color_type) {
+	if (xref && PNG_COLOR_TYPE_PALETTE & color_type) {
 		png_bytep trans;
 		int num_trans;
 		HPDF_Dict smask;
 		png_bytep smask_data;
 
-		if (!png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) || 
+		if (!png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) ||
 			!png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, NULL)) {
 			goto no_transparent_color_in_palette;
 		}
@@ -478,10 +489,10 @@ LoadPngData  (HPDF_Dict     image,
 		smask->header.obj_class |= HPDF_OSUBCLASS_XOBJECT;
 		ret = HPDF_Dict_AddName (smask, "Type", "XObject");
 		ret += HPDF_Dict_AddName (smask, "Subtype", "Image");
-		ret += HPDF_Dict_AddNumber (smask, "Width", (HPDF_UINT)info_ptr->width);
-		ret += HPDF_Dict_AddNumber (smask, "Height", (HPDF_UINT)info_ptr->height);
+		ret += HPDF_Dict_AddNumber (smask, "Width", (HPDF_UINT)width);
+		ret += HPDF_Dict_AddNumber (smask, "Height", (HPDF_UINT)height);
 		ret += HPDF_Dict_AddName (smask, "ColorSpace", "DeviceGray");
-		ret += HPDF_Dict_AddNumber (smask, "BitsPerComponent", (HPDF_UINT)info_ptr->bit_depth);
+		ret += HPDF_Dict_AddNumber (smask, "BitsPerComponent", (HPDF_UINT)bit_depth);
 
 		if (ret != HPDF_OK) {
 			HPDF_Dict_Free(smask);
@@ -489,7 +500,7 @@ LoadPngData  (HPDF_Dict     image,
 			goto Exit;
 		}
 
-		smask_data = HPDF_GetMem(image->mmgr, info_ptr->width * info_ptr->height);
+		smask_data = HPDF_GetMem(image->mmgr, width * height);
 		if (!smask_data) {
 			HPDF_Dict_Free(smask);
 			ret = HPDF_FAILD_TO_ALLOC_MEM;
@@ -503,7 +514,7 @@ LoadPngData  (HPDF_Dict     image,
 			goto Exit;
 		}
 
-		if (HPDF_Stream_Write(smask->stream, smask_data, info_ptr->width * info_ptr->height) != HPDF_OK) {
+		if (HPDF_Stream_Write(smask->stream, smask_data, width * height) != HPDF_OK) {
 			HPDF_FreeMem(image->mmgr, smask_data);
 			HPDF_Dict_Free(smask);
 			ret = HPDF_FILE_IO_ERROR;
@@ -513,9 +524,9 @@ LoadPngData  (HPDF_Dict     image,
 
 
 		ret += CreatePallet(image, png_ptr, info_ptr);
-		ret += HPDF_Dict_AddNumber (image, "Width", (HPDF_UINT)info_ptr->width);
-		ret += HPDF_Dict_AddNumber (image, "Height", (HPDF_UINT)info_ptr->height);
-		ret += HPDF_Dict_AddNumber (image, "BitsPerComponent",	(HPDF_UINT)info_ptr->bit_depth);
+		ret += HPDF_Dict_AddNumber (image, "Width", (HPDF_UINT)width);
+		ret += HPDF_Dict_AddNumber (image, "Height", (HPDF_UINT)height);
+		ret += HPDF_Dict_AddNumber (image, "BitsPerComponent",	(HPDF_UINT)bit_depth);
 		ret += HPDF_Dict_Add (image, "SMask", smask);
 
 		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -524,9 +535,9 @@ LoadPngData  (HPDF_Dict     image,
 
 no_transparent_color_in_palette:
 
-	/* read images with alpha channel right away 
+	/* read images with alpha channel right away
 	   we have to do this because image transparent mask must be added to the Xref */
-	if (xref && PNG_COLOR_MASK_ALPHA & info_ptr->color_type) {
+	if (xref && PNG_COLOR_MASK_ALPHA & color_type) {
 		HPDF_Dict smask;
 		png_bytep smask_data;
 
@@ -535,14 +546,16 @@ no_transparent_color_in_palette:
 			ret = HPDF_FAILD_TO_ALLOC_MEM;
 			goto Exit;
 		}
-
+		
+		smask->filter = image->filter;
 		smask->header.obj_class |= HPDF_OSUBCLASS_XOBJECT;
+		
 		ret = HPDF_Dict_AddName (smask, "Type", "XObject");
 		ret += HPDF_Dict_AddName (smask, "Subtype", "Image");
-		ret += HPDF_Dict_AddNumber (smask, "Width", (HPDF_UINT)info_ptr->width);
-		ret += HPDF_Dict_AddNumber (smask, "Height", (HPDF_UINT)info_ptr->height);
+		ret += HPDF_Dict_AddNumber (smask, "Width", (HPDF_UINT)width);
+		ret += HPDF_Dict_AddNumber (smask, "Height", (HPDF_UINT)height);
 		ret += HPDF_Dict_AddName (smask, "ColorSpace", "DeviceGray");
-		ret += HPDF_Dict_AddNumber (smask, "BitsPerComponent", (HPDF_UINT)info_ptr->bit_depth);
+		ret += HPDF_Dict_AddNumber (smask, "BitsPerComponent", (HPDF_UINT)bit_depth);
 
 		if (ret != HPDF_OK) {
 			HPDF_Dict_Free(smask);
@@ -550,7 +563,7 @@ no_transparent_color_in_palette:
 			goto Exit;
 		}
 
-		smask_data = HPDF_GetMem(image->mmgr, info_ptr->width * info_ptr->height);
+		smask_data = HPDF_GetMem(image->mmgr, width * height);
 		if (!smask_data) {
 			HPDF_Dict_Free(smask);
 			ret = HPDF_FAILD_TO_ALLOC_MEM;
@@ -564,7 +577,7 @@ no_transparent_color_in_palette:
 			goto Exit;
 		}
 
-		if (HPDF_Stream_Write(smask->stream, smask_data, info_ptr->width * info_ptr->height) != HPDF_OK) {
+		if (HPDF_Stream_Write(smask->stream, smask_data, width * height) != HPDF_OK) {
 			HPDF_FreeMem(image->mmgr, smask_data);
 			HPDF_Dict_Free(smask);
 			ret = HPDF_FILE_IO_ERROR;
@@ -572,14 +585,14 @@ no_transparent_color_in_palette:
 		}
 		HPDF_FreeMem(image->mmgr, smask_data);
 
-		if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
+		if (color_type == PNG_COLOR_TYPE_GRAY_ALPHA) {
 			ret += HPDF_Dict_AddName (image, "ColorSpace", "DeviceGray");
 		} else {
 			ret += HPDF_Dict_AddName (image, "ColorSpace", "DeviceRGB");
 		}
-		ret += HPDF_Dict_AddNumber (image, "Width", (HPDF_UINT)info_ptr->width);
-		ret += HPDF_Dict_AddNumber (image, "Height", (HPDF_UINT)info_ptr->height);
-		ret += HPDF_Dict_AddNumber (image, "BitsPerComponent",	(HPDF_UINT)info_ptr->bit_depth);
+		ret += HPDF_Dict_AddNumber (image, "Width", (HPDF_UINT)width);
+		ret += HPDF_Dict_AddNumber (image, "Height", (HPDF_UINT)height);
+		ret += HPDF_Dict_AddNumber (image, "BitsPerComponent",	(HPDF_UINT)bit_depth);
 		ret += HPDF_Dict_Add (image, "SMask", smask);
 
 		png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
@@ -589,9 +602,9 @@ no_transparent_color_in_palette:
 	/* if the image has color palette, copy the pallet of the image to
 	 * create color map.
 	 */
-	if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE)
+	if (color_type == PNG_COLOR_TYPE_PALETTE)
 		ret = CreatePallet(image, png_ptr, info_ptr);
-	else if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY)
+	else if (color_type == PNG_COLOR_TYPE_GRAY)
 		ret = HPDF_Dict_AddName (image, "ColorSpace", "DeviceGray");
 	else
 		ret = HPDF_Dict_AddName (image, "ColorSpace", "DeviceRGB");
@@ -617,16 +630,16 @@ no_transparent_color_in_palette:
 	}
 
 	/* setting the info of the image. */
-	if (HPDF_Dict_AddNumber (image, "Width", (HPDF_UINT)info_ptr->width)
+	if (HPDF_Dict_AddNumber (image, "Width", (HPDF_UINT)width)
 			!= HPDF_OK)
 		goto Exit;
 
-	if (HPDF_Dict_AddNumber (image, "Height", (HPDF_UINT)info_ptr->height)
+	if (HPDF_Dict_AddNumber (image, "Height", (HPDF_UINT)height)
 			!= HPDF_OK)
 		goto Exit;
 
 	if (HPDF_Dict_AddNumber (image, "BitsPerComponent",
-				(HPDF_UINT)info_ptr->bit_depth) != HPDF_OK)
+				(HPDF_UINT)bit_depth) != HPDF_OK)
 		goto Exit;
 
 	/* clean up */
@@ -695,5 +708,4 @@ PngAfterWrite  (HPDF_Dict obj)
 }
 
 
-#endif /* LIBHPDF_HAVE_NOPNGLIB */
-
+#endif /* LIBHPDF_HAVE_PNGLIB */
