@@ -14,20 +14,26 @@
  * It is provided "as is" without express or implied warranty.
  *
  */
+#ifndef _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_WARNINGS
+#endif
 
 #ifndef UNDER_CE
 #include <errno.h>
 #endif
+#ifndef HPDF_UNUSED
+#define HPDF_UNUSED(a) ((void)(a))
+#endif
 
 #include "hpdf_conf.h"
 #include "hpdf_consts.h"
 #include "hpdf_utils.h"
 #include "hpdf_streams.h"
 
-#ifndef LIBHPDF_HAVE_NOZLIB
+#ifdef LIBHPDF_HAVE_ZLIB
 #include <zlib.h>
 #include <zconf.h>
-#endif /* LIBHPDF_HAVE_NOZLIB */
+#endif /* LIBHPDF_HAVE_ZLIB */
 
 HPDF_STATUS
 HPDF_MemStream_WriteFunc  (HPDF_Stream      stream,
@@ -106,7 +112,7 @@ HPDF_FileStream_FreeFunc  (HPDF_Stream  stream);
  *  ptr : Pointer to a buffer to copy read data.
  *  size : Pointer to a variable which indecates buffer size.
  *
- *  HPDF_Stream_read returns HPDF_OK when success. On failer, it returns
+ *  HPDF_Stream_read returns HPDF_OK when success. On failure, it returns
  *  error-code returned by reading function of this stream.
  *
  */
@@ -135,7 +141,7 @@ HPDF_Stream_Read  (HPDF_Stream  stream,
  *  s : Pointer to a buffer to copy read data.
  *  size : buffer-size of s.
  *
- *  Read from stream until the buffer is exhausted or line-feed charactor is
+ *  Read from stream until the buffer is exhausted or line-feed character is
  *  read.
  *
  */
@@ -150,7 +156,10 @@ HPDF_Stream_ReadLn  (HPDF_Stream  stream,
 
     HPDF_PTRACE((" HPDF_Stream_ReadLn\n"));
 
-    if (!stream || !s || *size == 0)
+    if (!stream)
+        return HPDF_INVALID_PARAMETER;
+
+    if (!s || *size == 0)
         return HPDF_SetError (stream->error, HPDF_INVALID_PARAMETER, 0);
 
     if (!(stream->seek_fn) || !(stream->read_fn))
@@ -217,7 +226,7 @@ HPDF_Stream_ReadLn  (HPDF_Stream  stream,
  *  ptr : Pointer to a buffer to write.
  *  siz : The size of buffer to write.
  *
- *  HPDF_Stream_Write returns HPDF_OK when success. On failer, it returns
+ *  HPDF_Stream_Write returns HPDF_OK when success. On failure, it returns
  *  error-code returned by writing function of this stream.
  *
  */
@@ -434,18 +443,18 @@ HPDF_Stream_WriteEscapeText2  (HPDF_Stream    stream,
 
     buf[idx++] = '(';
 
-    for (i = 0; i < len; i++) {
+    for (i = 0; i < (HPDF_INT)len; i++) {
         HPDF_BYTE c = (HPDF_BYTE)*p++;
         if (HPDF_NEEDS_ESCAPE(c)) {
             buf[idx++] = '\\';
 
-            buf[idx] = c >> 6;
+            buf[idx] = (char)(c >> 6);
             buf[idx] += 0x30;
             idx++;
-            buf[idx] = (c & 0x38) >> 3;
+            buf[idx] = (char)((c & 0x38) >> 3);
             buf[idx] += 0x30;
             idx++;
-            buf[idx] = (c & 0x07);
+            buf[idx] = (char)(c & 0x07);
             buf[idx] += 0x30;
             idx++;
         }
@@ -511,7 +520,7 @@ HPDF_Stream_WriteBinary  (HPDF_Stream      stream,
     }
 
     for (i = 0; i < len; i++, p++) {
-        char c = *p >> 4;
+        char c = (char)(*p >> 4);
 
         if (c <= 9)
             c += 0x30;
@@ -519,7 +528,7 @@ HPDF_Stream_WriteBinary  (HPDF_Stream      stream,
             c += 0x41 - 10;
         buf[idx++] = c;
 
-        c = *p & 0x0f;
+        c = (char)(*p & 0x0f);
         if (c <= 9)
             c += 0x30;
         else
@@ -553,7 +562,7 @@ HPDF_Stream_WriteToStreamWithDeflate  (HPDF_Stream  src,
                                        HPDF_Stream  dst,
                                        HPDF_Encrypt  e)
 {
-#ifndef LIBHPDF_HAVE_NOZLIB
+#ifdef LIBHPDF_HAVE_ZLIB
 
 #define DEFLATE_BUF_SIZ  ((HPDF_INT)(HPDF_STREAM_BUF_SIZ * 1.1) + 13)
 
@@ -667,9 +676,12 @@ HPDF_Stream_WriteToStreamWithDeflate  (HPDF_Stream  src,
 
     deflateEnd(&strm);
     return HPDF_OK;
-#else /* LIBHPDF_HAVE_NOZLIB */
+#else /* LIBHPDF_HAVE_ZLIB */
+    HPDF_UNUSED (e);
+    HPDF_UNUSED (dst);
+    HPDF_UNUSED (src);
     return HPDF_UNSUPPORTED_FUNC;
-#endif /* LIBHPDF_HAVE_NOZLIB */
+#endif /* LIBHPDF_HAVE_ZLIB */
 }
 
 HPDF_STATUS
@@ -684,6 +696,7 @@ HPDF_Stream_WriteToStream  (HPDF_Stream  src,
     HPDF_BOOL flg;
 
     HPDF_PTRACE((" HPDF_Stream_WriteToStream\n"));
+    HPDF_UNUSED (filter);
 
     if (!dst || !(dst->write_fn)) {
         HPDF_SetError (src->error, HPDF_INVALID_OBJECT, 0);
@@ -698,10 +711,10 @@ HPDF_Stream_WriteToStream  (HPDF_Stream  src,
     if (HPDF_Stream_Size (src) == 0)
         return HPDF_OK;
 
-#ifndef LIBHPDF_HAVE_NOZLIB
+#ifdef LIBHPDF_HAVE_ZLIB
     if (filter & HPDF_STREAM_FILTER_FLATE_DECODE)
         return HPDF_Stream_WriteToStreamWithDeflate (src, dst, e);
-#endif /* LIBHPDF_HAVE_NOZLIB */
+#endif /* LIBHPDF_HAVE_ZLIB */
 
     ret = HPDF_Stream_Seek (src, 0, HPDF_SEEK_SET);
     if (ret != HPDF_OK)
@@ -823,7 +836,7 @@ HPDF_FileReader_ReadFunc  (HPDF_Stream  stream,
  *                     HPDF_SEEK_CUR : Relative to the current file position
  *                     HPDF_SEEK_END : Relative to the current end of file.
  *
- *  HPDF_FileReader_seek_fn returns HPDF_OK when successful. On failer
+ *  HPDF_FileReader_seek_fn returns HPDF_OK when successful. On failure
  *  the result is HPDF_FILE_IO_ERROR and HPDF_Error_GetCode2() returns the
  *  error which returned by file seeking function of platform.
  *
@@ -1088,7 +1101,7 @@ HPDF_MemStream_SeekFunc  (HPDF_Stream      stream,
     } else if (mode == HPDF_SEEK_END)
         pos = stream->size - pos;
 
-    if (pos > stream->size) {
+    if (pos > (HPDF_INT)stream->size) {
         return HPDF_SetError (stream->error, HPDF_STREAM_EOF, 0);
     }
 
@@ -1362,7 +1375,7 @@ HPDF_MemStream_Rewrite  (HPDF_Stream  stream,
 /*
  *  HPDF_CallbackReader_new
  *
- *  Constractor for HPDF_CallbackReader.
+ *  Constructor for HPDF_CallbackReader.
  *
  *  mmgr : Pointer to a HPDF_MMgr object.
  *  read_fn : Pointer to a user function for reading data.
@@ -1408,7 +1421,7 @@ HPDF_CallbackReader_New  (HPDF_MMgr              mmgr,
 /*
  *  HPDF_CallbackWriter_new
  *
- *  Constractor for HPDF_CallbackWriter.
+ *  Constructor for HPDF_CallbackWriter.
  *
  *  mmgr : Pointer to a HPDF_MMgr object.
  *  read_fn : Pointer to a user function for writing data.
